
/* renderer.canvas.export-image.js */

/**
 * This file is part of cytoscape.js 2.1.0.
 * 
 * Cytoscape.js is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Cytoscape.js is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with
 * cytoscape.js. If not, see <http://www.gnu.org/licenses/>.
 */
 
;(function($$){ "use strict";

	var CanvasRenderer = $$('renderer', 'canvas');

	CanvasRenderer.prototype.createBuffer = function(w, h) {
		var buffer = document.createElement("canvas");
		buffer.width = w;
		buffer.height = h;
		
		return [buffer, buffer.getContext("2d")];
	}

	CanvasRenderer.prototype.png = function( options ){
		var data = this.data;
		var cy = data.cy;
		var bb = cy.boundingBox();
		var width = options.full ? Math.ceil(bb.w) : this.data.container.clientWidth;
		var height = options.full ? Math.ceil(bb.h) : this.data.container.clientHeight;
		var buffCanvas = document.createElement("canvas");

		buffCanvas.width = width;
		buffCanvas.height = height;

		buffCanvas.style.width = width + 'px';
		buffCanvas.style.height = height + 'px';

		var buffCxt = buffCanvas.getContext("2d");

		// Rasterize the layers, but only if container has nonzero size
		if (width > 0 && height > 0) {

			buffCxt.clearRect( 0, 0, width, height );

			if( options.bg ){
				buffCxt.fillStyle = options.bg;
				buffCxt.rect( 0, 0, width, height );
				buffCxt.fill();
			}

			buffCxt.globalCompositeOperation = "source-over";

			if( options.full ){ // draw the full bounds of the graph
				this.redraw({
					forcedContext: buffCxt,
					drawAllLayers: true,
					forcedZoom: 1,
					forcedPan: { x: -bb.x1, y: -bb.y1 }
				});
			} else { // draw the current view
				this.redraw({
					forcedContext: buffCxt,
					drawAllLayers: true
				});
			}
		}

		return buffCanvas.toDataURL("image/png");
	};

})( cytoscape );